@@ -3,7 +3,7 @@
 #include <errno.h>
 
 #include <skalibs/sgetopt.h>
-#include <skalibs/strerr2.h>
+#include <skalibs/strerr.h>
 #include <skalibs/stralloc.h>
 #include <skalibs/env.h>
 #include <skalibs/exec.h>
@@ -36,6 +36,6 @@ int main (int argc, char const *const *argv)
   argc -= l.ind ; argv += l.ind ;
   if (argc < 2) strerr_dieusage(100, USAGE) ;
   if ((envdir_internal(*argv++, &modifs, options, nullis) < 0) && (insist || (errno != ENOENT)))
-    strerr_diefu1sys(111, "envdir") ;
+    strerr_diefu2sys(111, "envdir ", argv[-1]) ;
   xmexec_m(argv, modifs.s, modifs.len) ;
 }
