@@ -300,10 +300,10 @@ static int do_retrieve (uint32_t cc, unixmessage const *m)
   return 1 ;
 }
 
-static int fill_siovec_with_ids_iter (char *thing, void *data)
+static int fill_siovec_with_ids_iter (void *thing, void *data)
 {
   struct iovec *v = (*(struct iovec **)data)++ ;
-  s6_fdholder_fd_t *p = (s6_fdholder_fd_t *)thing ;
+  s6_fdholder_fd_t *p = thing ;
   v->iov_base = p->id ;
   v->iov_len = strlen(p->id) + 1 ;
   return 1 ;
@@ -334,9 +334,9 @@ struct getdumpiter_s
   char *limit ;
 } ;
 
-static int getdump_iter (char *thing, void *stuff)
+static int getdump_iter (void *thing, void *stuff)
 {
-  s6_fdholder_fd_t *p = (s6_fdholder_fd_t *)thing ;
+  s6_fdholder_fd_t *p = thing ;
   getdumpiter_t *blah = stuff ;
   unsigned char len = strlen(p->id) ;
   tain_pack(blah->limit, &p->limit) ;
@@ -645,7 +645,6 @@ static inline int new_connection (int fd, regex_t *rre, regex_t *wre, unsigned i
 
 int main (int argc, char const *const *argv, char const *const *envp)
 {
-  int spfd ;
   int flag1 = 0 ;
   uint32_t maxconn = 16 ;
   PROG = "s6-fdholderd" ;
@@ -700,8 +699,7 @@ int main (int argc, char const *const *argv, char const *const *envp)
       strerr_dief1sys(100, "called with option -1 but stdout said") ;
   }
   else close(1) ;
-  spfd = selfpipe_init() ;
-  if (spfd < 0) strerr_diefu1sys(111, "selfpipe_init") ;
+  if (selfpipe_init() == -1) strerr_diefu1sys(111, "selfpipe_init") ;
   if (!sig_ignore(SIGPIPE)) strerr_diefu1sys(111, "ignore SIGPIPE") ;
   {
     sigset_t set ;
@@ -743,7 +741,7 @@ int main (int argc, char const *const *argv, char const *const *envp)
     avltreen_init(&fddeadlineinfo, fddeadlinestorage, fddeadlinefreelist, maxfds, &fds_deadline_dtok, &fds_deadline_cmp, 0) ;
     fds_by_deadline = &fddeadlineinfo ;
 
-    x[0].fd = spfd ; x[0].events = IOPAUSE_READ ;
+    x[0].fd = selfpipe_fd() ; x[0].events = IOPAUSE_READ ;
     x[1].fd = 0 ;
       
     if (flag1)
