@@ -13,7 +13,9 @@
 #include <skalibs/error.h>
 #include <skalibs/strerr.h>
 #include <skalibs/buffer.h>
+#include <skalibs/alloc.h>
 #include <skalibs/stralloc.h>
+#include <skalibs/genalloc.h>
 #include <skalibs/sig.h>
 #include <skalibs/tai.h>
 #include <skalibs/djbunix.h>
@@ -27,17 +29,16 @@
 #include <skalibs/posixishard.h>
 
 #define FTRIGRD_MAXREADS 32
-#define FTRIGRD_BUFSIZE 17
+#define FTRIGRD_BUFSIZE 16
 
 #define dienomem() strerr_diefu1sys(111, "stralloc_catb")
 
-typedef struct ftrigio_s ftrigio_t, *ftrigio_t_ref ;
+typedef struct ftrigio_s ftrigio, *ftrigio_ref ;
 struct ftrigio_s
 {
   unsigned int xindex ;
   ftrig1_t trig ;
   buffer b ;
-  char buf[FTRIGRD_BUFSIZE] ;
   regex_t re ;
   stralloc sa ;
   uint32_t options ;
@@ -45,11 +46,11 @@ struct ftrigio_s
 } ;
 #define FTRIGIO_ZERO { .xindex = 0, .trig = FTRIG1_ZERO, .b = BUFFER_INIT(0, -1, 0, 0), .buf = "", .sa = STRALLOC_ZERO, .options = 0, .id = 0 }
 
-static ftrigio_t a[FTRIGR_MAX] ;
-static unsigned int n = 0 ;
+static genalloc g = GENALLOC_ZERO ; /* ftrigio */
 
-static void ftrigio_deepfree (ftrigio_t *p)
+static void ftrigio_free (ftrigio *p)
 {
+  alloc_free(p->b.c.x) ;
   ftrig1_free(&p->trig) ;
   stralloc_free(&p->sa) ;
   regfree(&p->re) ;
@@ -57,9 +58,10 @@ static void ftrigio_deepfree (ftrigio_t *p)
 
 static void cleanup (void)
 {
-  unsigned int i = 0 ;
-  for (; i < n ; i++) ftrigio_deepfree(a + i) ;
-  n = 0 ;
+  size_t n = genalloc_len(ftrigio, &g) ;
+  ftrigio *a = genalloc_s(ftrigio, &g) ;
+  for (size_t i = 0 ; i < n ; i++) ftrigio_free(a + i) ;
+  genalloc_setlen(ftrigio, &g, 0) ;
 }
 
 static void trig (uint16_t id, char what, char info)
@@ -83,14 +85,16 @@ static void answer (unsigned char c)
   }
 }
 
-static void remove (unsigned int i)
+static void remove (size_t i)
 {
-  ftrigio_deepfree(a + i) ;
+  size_t n = genalloc_len(ftrigio, &g) ;
+  ftrigio *a = genalloc_s(ftrigio, &g) ;
+  ftrigio_free(a + i) ;
   a[i] = a[--n] ;
-  a[i].b.c.x = a[i].buf ;
+  genalloc_setlen(ftrigio, &g, n) ;
 }
 
-static inline int ftrigio_read (ftrigio_t *p)
+static inline int ftrigio_read (ftrigio *p)
 {
   unsigned int i = FTRIGRD_MAXREADS ;
   while (i--)
@@ -101,7 +105,11 @@ static inline int ftrigio_read (ftrigio_t *p)
     if (!r) break ;
     if (r < 0) return (trig(p->id, 'd', errno), 0) ;
     blen = buffer_len(&p->b) ;
-    if (!stralloc_readyplus(&p->sa, blen+1)) dienomem() ;
+    if (!stralloc_readyplus(&p->sa, blen+1))
+    {
+      cleanup() ;
+      dienomem() ;
+    }
     buffer_getnofill(&p->b, p->sa.s + p->sa.len, blen) ;
     p->sa.len += blen ;
     p->sa.s[p->sa.len] = 0 ;
@@ -130,8 +138,9 @@ static int parse_protocol (struct iovec const *v, void *context)
   {
     case 'U' : /* unsubscribe */
     {
-      unsigned int i = 0 ;
-      for (; i < n ; i++) if (a[i].id == id) break ;
+      size_t n = genalloc_len(ftrigio, &g) ;
+      size_t i = 0 ;
+      for (; i < n ; i++) if (genalloc_s(ftrigio, &g)[i].id == id) break ;
       if (i < n)
       {
         remove(i) ;
@@ -142,6 +151,9 @@ static int parse_protocol (struct iovec const *v, void *context)
     }
     case 'L' : /* subscribe to path and match re */
     {
+      size_t n = genalloc_len(ftrigio, &g) ;
+      ftrigio *p ;
+      char *x ;
       uint32_t options, pathlen, relen ;
       int r ;
       if (v->iov_len < 19)
@@ -157,36 +169,43 @@ static int parse_protocol (struct iovec const *v, void *context)
         answer(EPROTO) ;
         break ;
       }
-      if (n >= FTRIGR_MAX)
+      if (!genalloc_readyplus(ftrigio, &g, 1))
       {
-        answer(ENFILE) ;
+        answer(ENOMEM) ;
         break ;
       }
-      r = skalibs_regcomp(&a[n].re, s + 16 + pathlen, REG_EXTENDED) ;
+      x = alloc(FTRIGRD_BUFSIZE) ;
+      if (!x)
+      {
+        answer(ENOMEM) ;
+        break ;
+      }
+      p = genalloc_s(ftrigio, &g) + n ;
+      r = skalibs_regcomp(&p->re, s + 16 + pathlen, REG_EXTENDED) ;
       if (r)
       {
+        alloc_free(x) ;
         answer(r == REG_ESPACE ? ENOMEM : EINVAL) ;
         break ;
       }
-      if (!ftrig1_make(&a[n].trig, s + 15))
+      if (!ftrig1_make(&p->trig, s + 15))
       {
-        regfree(&a[n].re) ;
+        regfree(&p->re) ;
+        alloc_free(x) ;
         answer(errno) ;
         break ;
       }
-      buffer_init(&a[n].b, &buffer_read, a[n].trig.fd, a[n].buf, FTRIGRD_BUFSIZE) ;
-      a[n].options = options ;
-      a[n].id = id ;
-      a[n].sa = stralloc_zero ;
-      n++ ;
+      buffer_init(&p->b, &buffer_read, p->trig.fd, x, FTRIGRD_BUFSIZE) ;
+      p->options = options ;
+      p->id = id ;
+      p->sa = stralloc_zero ;
+      genalloc_setlen(ftrigio, &g, n+1) ;
       answer(0) ;
       break ;
     }
     default :
-    {
       cleanup() ;
       strerr_dief1x(100, "invalid client request") ;
-    }
   }
   (void)context ;
   return 1 ;
@@ -196,9 +215,10 @@ int main (void)
 {
   PROG = "s6-ftrigrd" ;
 
-  if (ndelay_on(0) < 0) strerr_diefu2sys(111, "ndelay_on ", "0") ;
-  if (ndelay_on(1) < 0) strerr_diefu2sys(111, "ndelay_on ", "1") ;
-  if (!sig_ignore(SIGPIPE)) strerr_diefu1sys(111, "ignore SIGPIPE") ;
+  if (ndelay_on(0) == -1 || ndelay_on(1) == -1)
+    strerr_diefu1sys(111, "make fds nonblocking") ;
+  if (!sig_altignore(SIGPIPE))
+    strerr_diefu1sys(111, "ignore SIGPIPE") ;
 
   {
     tain deadline ;
@@ -210,8 +230,9 @@ int main (void)
 
   for (;;)
   {
+    size_t n = genalloc_len(ftrigio, &g) ;
+    size_t i = 0 ;
     iopause_fd x[3 + n] ;
-    unsigned int i = 0 ;
 
     x[0].fd = 0 ; x[0].events = IOPAUSE_EXCEPT | IOPAUSE_READ ;
     x[1].fd = 1 ; x[1].events = IOPAUSE_EXCEPT | (textmessage_sender_isempty(textmessage_sender_1) ? 0 : IOPAUSE_WRITE) ;
@@ -219,8 +240,9 @@ int main (void)
     x[2].events = IOPAUSE_EXCEPT | (textmessage_sender_isempty(textmessage_sender_x) ? 0 : IOPAUSE_WRITE) ;
     for (; i < n ; i++)
     {
-      a[i].xindex = 3 + i ;
-      x[3+i].fd = a[i].trig.fd ;
+      ftrigio *p = genalloc_s(ftrigio, &g) + i ;
+      p->xindex = 3+i ;
+      x[3+i].fd = p->trig.fd ;
       x[3+i].events = IOPAUSE_READ ;
     }
 
@@ -248,10 +270,11 @@ int main (void)
       }
 
    /* scan listening ftrigs */
-    for (i = 0 ; i < n ; i++)
+    for (i = 0 ; i < genalloc_len(ftrigio, &g) ; i++)
     {
-      if (x[a[i].xindex].revents & IOPAUSE_READ)
-        if (!ftrigio_read(a+i)) remove(i--) ;
+      ftrigio *p = genalloc_s(ftrigio, &g) + i ;
+      if (x[p->xindex].revents & IOPAUSE_READ)
+        if (!ftrigio_read(p)) remove(i--) ;
     }
 
    /* client is writing */
