@@ -2,7 +2,7 @@
 
 #include <skalibs/djbunix.h>
 #include <skalibs/unixmessage.h>
-#include <s6/s6-fdholder.h>
+#include <s6/fdholder.h>
 
 void s6_fdholder_end (s6_fdholder_t *a)
 {
