@@ -5,7 +5,7 @@
 
 #include <skalibs/djbunix.h>
 
-#include <s6/s6-supervise.h>
+#include <s6/supervise.h>
 
 int s6_svc_ok (char const *dir)
 {
